# -*- coding: UTF8 -*-
"""
    MoinMoin - ... macro

    Allows to ... .

    @copyright: 20??-20?? Author Name <author@url.org>,
    @license: GNU GPL, see COPYING for details.
"""

import os

from MoinMoin import wikiutil
from MoinMoin.Page import Page
from MoinMoin.parser.text_moin_wiki import Parser as MoinParser

plugin_name = os.path.splitext(os.path.basename(__file__))[0]


class Macro:
    """ Description what your parser does.
    """
    def __init__(self, macro, needle):
        self.request = macro.request
        self.pagename = self.request.page.page_name
        self.formatter = self.request.formatter
        self.page = Page(self.request, self.pagename)
        self.pagepath = self.page.getPageBasePath()[1]
        self.url = self.request.href(self.request.page.page_name)
        self.url_full = self.request.getQualifiedURL(self.url)
        self.title = self.page.split_title()
        self.formargs = self.request.values
        self.get_all_my_page_text = wikiutil.importPlugin(self.request.cfg, 'parser', 'template', 'get_all_my_page_text')
        self.settings = wikiutil.importPlugin(self.request.cfg, 'parser', 'template', 'settings')
        update_defaults_by_wikiconfig = wikiutil.importPlugin(self.request.cfg, 'parser', 'schedule', 'update_defaults_by_wikiconfig')
        update_defaults_by_wikiconfig(self.request.cfg, plugin_name, self.settings)
        self.error = u''
        self.no_error = self.error

        try:
            my_settings = wikiutil.invoke_extension_function(self.request, self.settings, needle)
            for key, value in my_settings.items():
                setattr(self, key, value)
            # saves the state of valid input
        except ValueError, err:
            self.error += "/!\\ " + str(err) + " /!\\\n"
        else:
            pass  # your functionality from here on

    def do(self):
        """ Send the "macro" text.
        """
        if self.error != self.no_error:
            moin_output = self.error
        else:
            moin_output = """
== Page Information ==
 * Moin-Pagename: %s
 * Moin-Pagetitle: %s
 * Moin-Pagepath: %s
 * Page-Url: %s
 * Full Page-Url: %s

== Url/ Form data ==
""" % (self.pagename, self.title, self.pagepath, self.url, self.url_full)
            for key in self.formargs.keys():
                moin_output += u" * %s=%s\n" % (key, self.formargs.get(key))
            moin_output += "== Acl ==\n"
            if self.request.user.may.read(self.pagename):
                moin_output += u" * You have read rights.\n"
            else:
                moin_output += " * You don't have read rights.\n"
            if self.request.user.may.write(self.pagename):
                moin_output += u" * You have write rights.\n"
            else:
                moin_output += u" * You don't have write rights.\n"
            moin_output += "== Settings ==\n"
            for key in self.settings().keys():
                try:
                    moin_output += u" * %s=%s\n" % (key, str(getattr(self, key)))
                except:
                    moin_output += u" * %s= /!\\ Not existing /!\\\n" % (key)
            moin_output += "== Wiki Content for the parser of this macro ==\n"
            tc = self.get_all_my_page_text(self.request, self.subpage_of)
            for page_name in tc:
                moin_output += "=== %s ===\n" % page_name
                for wiki_text in tc[page_name]:
                    moin_output += u"{{{\n%s\n}}}\n" % (wiki_text)
        parser = MoinParser(moin_output, self.request)
        parser.format(self.formatter)
        return ''


def execute(macro, needle):
    """ Function which is called by MoinMoin when this macro is needed.
        @param macro: macro object of MoinMoin
        @param needle: setting text of the macro e.g. "test=1" when <<macroname(test=1)>> is used
    """
    return Macro(macro, needle).do()
